@@ -10,9 +10,9 @@
 #
 # $Project: Convert-Binary-C $
 # $Author: mhx $
-# $Date: 2002/10/12 16:59:27 +0200 $
-# $Revision: 5 $
-# $Snapshot: /Convert-Binary-C/0.05 $
+# $Date: 2002/12/11 14:48:16 +0100 $
+# $Revision: 7 $
+# $Snapshot: /Convert-Binary-C/0.06 $
 # $Source: /ctlib/Tokenizer.pm $
 #
 ################################################################################
@@ -27,7 +27,7 @@ package Tokenizer;
 use strict;
 use vars '$VERSION';
 
-$VERSION = sprintf '%.2f', 0.01*('$Revision: 5 $' =~ /(\d+)/)[0];
+$VERSION = sprintf '%.2f', 0.01*('$Revision: 7 $' =~ /(\d+)/)[0];
 
 sub new
 {
@@ -145,8 +145,11 @@ Tokenizer - Generate C source for fast keyword tokenizer
 The Tokenizer module provides a small class for creating the
 essential ANSI C source code for a fast keyword tokenizer.
 
-The code created by the above example would print the following
-C source code:
+The generated code is optimized for speed. On the ANSI-C
+keyword set, it's 2-3 times faster than equivalent code
+generated with the C<gprof> utility.
+
+The above example would print the following C source code:
 
   switch( tokstr[0] )
   {
